@@ -6,16 +6,18 @@ from dcim.models import Device, Interface, Region, Site, SiteGroup, VirtualChass
 from django import forms
 from django.contrib.contenttypes.models import ContentType
 from django.utils.safestring import mark_safe
-from extras.models import Tag
 from ipam.models import Prefix
 from netbox.forms import NetBoxModelForm
-from utilities.forms import (
-    CommentField,
-    DynamicModelChoiceField,
-    DynamicModelMultipleChoiceField,
+from utilities.forms import CommentField, DynamicModelChoiceField
+from virtualization.models import (
+    Cluster,
+    ClusterGroup,
+    ClusterType,
+    VirtualMachine,
+    VMInterface,
 )
-from virtualization.models import VirtualMachine, VMInterface
 
+from ..choices import ACLTypeChoices
 from ..models import (
     AccessList,
     ACLExtendedRule,
@@ -59,6 +61,7 @@ class AccessListForm(NetBoxModelForm):
     Requires a device, a name, a type, and a default_action.
     """
 
+    # Device selector
     region = DynamicModelChoiceField(
         queryset=Region.objects.all(),
         required=False,
@@ -85,21 +88,49 @@ class AccessListForm(NetBoxModelForm):
             "site_id": "$site",
         },
     )
+
+    # Virtual Chassis selector
     virtual_chassis = DynamicModelChoiceField(
         queryset=VirtualChassis.objects.all(),
         required=False,
         label="Virtual Chassis",
     )
+
+    # Virtual Machine selector
+    cluster_type = DynamicModelChoiceField(
+        queryset=ClusterType.objects.all(),
+        required=False,
+    )
+
+    cluster_group = DynamicModelChoiceField(
+        queryset=ClusterGroup.objects.all(),
+        required=False,
+        query_params={
+            "type_id": "$cluster_type",
+        },
+    )
+
+    cluster = DynamicModelChoiceField(
+        queryset=Cluster.objects.all(),
+        required=False,
+        query_params={
+            "type_id": "$cluster_type",
+            "group_id": "$cluster_group",
+        },
+    )
+
     virtual_machine = DynamicModelChoiceField(
         queryset=VirtualMachine.objects.all(),
         required=False,
         label="Virtual Machine",
+        query_params={
+            "cluster_type_id": "$cluster_type",
+            "cluster_group_id": "$cluster_group",
+            "cluster_id": "$cluster",
+        },
     )
+
     comments = CommentField()
-    tags = DynamicModelMultipleChoiceField(
-        queryset=Tag.objects.all(),
-        required=False,
-    )
 
     class Meta:
         model = AccessList
@@ -176,18 +207,18 @@ class AccessListForm(NetBoxModelForm):
         if device:
             host_type = "device"
             existing_acls = AccessList.objects.filter(name=name, device=device).exists()
-        elif virtual_chassis:
-            host_type = "virtual_chassis"
-            existing_acls = AccessList.objects.filter(
-                name=name,
-                virtual_chassis=virtual_chassis,
-            ).exists()
         elif virtual_machine:
             host_type = "virtual_machine"
             existing_acls = AccessList.objects.filter(
                 name=name,
                 virtual_machine=virtual_machine,
             ).exists()
+        else:
+            host_type = "virtual_chassis"
+            existing_acls = AccessList.objects.filter(
+                name=name,
+                virtual_chassis=virtual_chassis,
+            ).exists()
 
         # Check if duplicate entry.
         if (
@@ -200,13 +231,18 @@ class AccessListForm(NetBoxModelForm):
                 host_type: [error_same_acl_name],
                 "name": [error_same_acl_name],
             }
-        # Check if Access List has no existing rules before change the Access List's type.
-        if (acl_type == "extended" and self.instance.aclstandardrules.exists()) or (
-            acl_type == "standard" and self.instance.aclextendedrules.exists()
-        ):
-            error_message["type"] = [
-                "This ACL has ACL rules associated, CANNOT change ACL type.",
-            ]
+        if self.instance.pk:
+            # Check if Access List has no existing rules before change the Access List's type.
+            if (
+                acl_type == ACLTypeChoices.TYPE_EXTENDED
+                and self.instance.aclstandardrules.exists()
+            ) or (
+                acl_type == ACLTypeChoices.TYPE_STANDARD
+                and self.instance.aclextendedrules.exists()
+            ):
+                error_message["type"] = [
+                    "This ACL has ACL rules associated, CANNOT change ACL type.",
+                ]
 
         if error_message:
             raise forms.ValidationError(error_message)
@@ -233,9 +269,9 @@ class ACLInterfaceAssignmentForm(NetBoxModelForm):
     device = DynamicModelChoiceField(
         queryset=Device.objects.all(),
         required=False,
-        query_params={
-            # Need to pass ACL device to it
-        },
+        # query_params={
+        # Need to pass ACL device to it
+        # },
     )
     interface = DynamicModelChoiceField(
         queryset=Interface.objects.all(),
@@ -247,6 +283,9 @@ class ACLInterfaceAssignmentForm(NetBoxModelForm):
     virtual_machine = DynamicModelChoiceField(
         queryset=VirtualMachine.objects.all(),
         required=False,
+        # query_params={
+        # Need to pass ACL device to it
+        # },
         label="Virtual Machine",
     )
     vminterface = DynamicModelChoiceField(
@@ -274,10 +313,6 @@ class ACLInterfaceAssignmentForm(NetBoxModelForm):
         ),
     )
     comments = CommentField()
-    tags = DynamicModelMultipleChoiceField(
-        queryset=Tag.objects.all(),
-        required=False,
-    )
 
     def __init__(self, *args, **kwargs):
 
@@ -317,7 +352,7 @@ class ACLInterfaceAssignmentForm(NetBoxModelForm):
         """
         Validates form inputs before submitting:
           - Check if both interface and vminterface are set.
-          - Check if neither interface or vminterface are set.
+          - Check if neither interface nor vminterface are set.
           - Check that an interface's parent device/virtual_machine is assigned to the Access List.
           - Check that an interface's parent device/virtual_machine is assigned to the Access List.
           - Check for duplicate entry. (Because of GFK)
@@ -329,35 +364,14 @@ class ACLInterfaceAssignmentForm(NetBoxModelForm):
         direction = cleaned_data.get("direction")
         interface = cleaned_data.get("interface")
         vminterface = cleaned_data.get("vminterface")
-        assigned_object = cleaned_data.get("assigned_object")
-        if interface:
-            assigned_object = interface
-            assigned_object_type = "interface"
-            host_type = "device"
-            host = Interface.objects.get(pk=assigned_object.pk).device
-        elif vminterface:
-            assigned_object = vminterface
-            assigned_object_type = "vminterface"
-            host_type = "virtual_machine"
-            host = VMInterface.objects.get(pk=assigned_object.pk).virtual_machine
-        if interface or vminterface:
-            assigned_object_id = VMInterface.objects.get(pk=assigned_object.pk).pk
-            assigned_object_type_id = ContentType.objects.get_for_model(
-                assigned_object,
-            ).pk
-        access_list_host = AccessList.objects.get(pk=access_list.pk).assigned_object
 
         # Check if both interface and vminterface are set.
         if interface and vminterface:
             error_too_many_interfaces = "Access Lists must be assigned to one type of interface at a time (VM interface or physical interface)"
-            error_too_many_hosts = "Access Lists must be assigned to one type of device at a time (VM or physical device)."
             error_message |= {
-                "device": [error_too_many_hosts],
                 "interface": [error_too_many_interfaces],
-                "virtual_machine": [error_too_many_hosts],
                 "vminterface": [error_too_many_interfaces],
             }
-        # Check if neither interface or vminterface are set.
         elif not (interface or vminterface):
             error_no_interface = (
                 "An Access List assignment but specify an Interface or VM Interface."
@@ -366,40 +380,61 @@ class ACLInterfaceAssignmentForm(NetBoxModelForm):
                 "interface": [error_no_interface],
                 "vminterface": [error_no_interface],
             }
-        # Check that an interface's parent device/virtual_machine is assigned to the Access List.
-        elif access_list_host != host:
-            error_acl_not_assigned_to_host = "Access List not present on selected host."
-            error_message |= {
-                "access_list": [error_acl_not_assigned_to_host],
-                assigned_object_type: [error_acl_not_assigned_to_host],
-                host_type: [error_acl_not_assigned_to_host],
-            }
-        # Check for duplicate entry.
-        elif ACLInterfaceAssignment.objects.filter(
-            access_list=access_list,
-            assigned_object_id=assigned_object_id,
-            assigned_object_type=assigned_object_type_id,
-            direction=direction,
-        ).exists():
-            error_duplicate_entry = "An ACL with this name is already associated to this interface & direction."
-            error_message |= {
-                "access_list": [error_duplicate_entry],
-                "direction": [error_duplicate_entry],
-                assigned_object_type: [error_duplicate_entry],
-            }
-        # Check that the interface does not have an existing ACL applied in the direction already.
-        elif ACLInterfaceAssignment.objects.filter(
-            assigned_object_id=assigned_object_id,
-            assigned_object_type=assigned_object_type_id,
-            direction=direction,
-        ).exists():
-            error_interface_already_assigned = (
-                "Interfaces can only have 1 Access List assigned in each direction."
-            )
-            error_message |= {
-                "direction": [error_interface_already_assigned],
-                assigned_object_type: [error_interface_already_assigned],
-            }
+        else:
+            if interface:
+                assigned_object = interface
+                assigned_object_type = "interface"
+                host_type = "device"
+                host = Interface.objects.get(pk=assigned_object.pk).device
+                assigned_object_id = Interface.objects.get(pk=assigned_object.pk).pk
+            else:
+                assigned_object = vminterface
+                assigned_object_type = "vminterface"
+                host_type = "virtual_machine"
+                host = VMInterface.objects.get(pk=assigned_object.pk).virtual_machine
+                assigned_object_id = VMInterface.objects.get(pk=assigned_object.pk).pk
+
+            assigned_object_type_id = ContentType.objects.get_for_model(
+                assigned_object,
+            ).pk
+            access_list_host = AccessList.objects.get(pk=access_list.pk).assigned_object
+
+            # Check that an interface's parent device/virtual_machine is assigned to the Access List.
+            if access_list_host != host:
+                error_acl_not_assigned_to_host = (
+                    "Access List not present on selected host."
+                )
+                error_message |= {
+                    "access_list": [error_acl_not_assigned_to_host],
+                    assigned_object_type: [error_acl_not_assigned_to_host],
+                    host_type: [error_acl_not_assigned_to_host],
+                }
+            # Check for duplicate entry.
+            if ACLInterfaceAssignment.objects.filter(
+                access_list=access_list,
+                assigned_object_id=assigned_object_id,
+                assigned_object_type=assigned_object_type_id,
+                direction=direction,
+            ).exists():
+                error_duplicate_entry = "An ACL with this name is already associated to this interface & direction."
+                error_message |= {
+                    "access_list": [error_duplicate_entry],
+                    "direction": [error_duplicate_entry],
+                    assigned_object_type: [error_duplicate_entry],
+                }
+            # Check that the interface does not have an existing ACL applied in the direction already.
+            if ACLInterfaceAssignment.objects.filter(
+                assigned_object_id=assigned_object_id,
+                assigned_object_type=assigned_object_type_id,
+                direction=direction,
+            ).exists():
+                error_interface_already_assigned = (
+                    "Interfaces can only have 1 Access List assigned in each direction."
+                )
+                error_message |= {
+                    "direction": [error_interface_already_assigned],
+                    assigned_object_type: [error_interface_already_assigned],
+                }
 
         if error_message:
             raise forms.ValidationError(error_message)
@@ -410,7 +445,6 @@ class ACLInterfaceAssignmentForm(NetBoxModelForm):
         self.instance.assigned_object = self.cleaned_data.get(
             "interface",
         ) or self.cleaned_data.get("vminterface")
-
         return super().save(*args, **kwargs)
 
 
@@ -424,7 +458,7 @@ class ACLStandardRuleForm(NetBoxModelForm):
     access_list = DynamicModelChoiceField(
         queryset=AccessList.objects.all(),
         query_params={
-            "type": "standard",
+            "type": ACLTypeChoices.TYPE_STANDARD,
         },
         help_text=mark_safe(
             "<b>*Note:</b> This field will only display Standard ACLs.",
@@ -437,10 +471,6 @@ class ACLStandardRuleForm(NetBoxModelForm):
         help_text=help_text_acl_rule_logic,
         label="Source Prefix",
     )
-    tags = DynamicModelMultipleChoiceField(
-        queryset=Tag.objects.all(),
-        required=False,
-    )
 
     fieldsets = (
         ("Access List Details", ("access_list", "description", "tags")),
@@ -506,17 +536,14 @@ class ACLExtendedRuleForm(NetBoxModelForm):
     access_list = DynamicModelChoiceField(
         queryset=AccessList.objects.all(),
         query_params={
-            "type": "extended",
+            "type": ACLTypeChoices.TYPE_EXTENDED,
         },
         help_text=mark_safe(
             "<b>*Note:</b> This field will only display Extended ACLs.",
         ),
         label="Access List",
     )
-    tags = DynamicModelMultipleChoiceField(
-        queryset=Tag.objects.all(),
-        required=False,
-    )
+
     source_prefix = DynamicModelChoiceField(
         queryset=Prefix.objects.all(),
         required=False,
